#include "StdAfx.h"
#include "FIXMessageEventHandler.h"

const static char version[32] = "FastDecoder v1.1.4.0";

namespace FAST
{

CFIXMessageEventHandlerContainer::CFIXMessageEventHandlerContainer(void)
{
	std::string v = version;
	if( v.empty() ) {
		v.clear();
	}
}

CFIXMessageEventHandlerContainer::~CFIXMessageEventHandlerContainer(void)
{
}

// Register/Unregister
bool CFIXMessageEventHandlerContainer::registerEventHandler(IFIXMessageEventHandlerPtr fixMessageEventHandlerPtr)
{
	UINT_PTR key = reinterpret_cast<UINT_PTR>(fixMessageEventHandlerPtr.get());
	TFIXMessageEventHandlerMap::iterator it = this->m_fixMessageEventHandlerMap.find(key);
	if (it != this->m_fixMessageEventHandlerMap.end())
		return false;

	this->m_fixMessageEventHandlerMap.insert(TFIXMessageEventHandlerMap::value_type(key, fixMessageEventHandlerPtr));
	return true;
}

bool CFIXMessageEventHandlerContainer::unregisterEventHandler(IFIXMessageEventHandlerPtr fixMessageEventHandlerPtr)
{
	return this->m_fixMessageEventHandlerMap.erase(reinterpret_cast<UINT_PTR>(fixMessageEventHandlerPtr.get())) ? true : false;
}

#ifdef _DUMP_MESSAGES
#define DUMP_SS(_ss) \
	char buf[_MAX_PATH]; \
	sprintf_s(buf, sizeof(buf), "D:\\Thiago\\LOGS\\%1d_DECODER_%1d.log", _sourceID, _channelID); \
	FILE *f = fopen(buf, "ab+"); if( f != NULL ) { fprintf(f, "%s", _ss.str().c_str()); fflush(f); fclose(f); }
#endif

// Event callbacks
void CFIXMessageEventHandlerContainer::onMessageBegin(int channelID, unsigned int sourceID)
{
#ifdef _DUMP_MESSAGES
	_channelID = channelID;
	_sourceID = sourceID;

	std::stringstream _ss;
	_ss << "BEGIN c:" << channelID << "s:" << sourceID << " [";
	DUMP_SS(_ss);
#endif

	TFIXMessageEventHandlerMap::iterator it;
	for (it = this->m_fixMessageEventHandlerMap.begin(); it != this->m_fixMessageEventHandlerMap.end(); ++it)
		it->second->onMessageBegin(channelID, sourceID);
}

void CFIXMessageEventHandlerContainer::onMessageEnd(int channelID, unsigned int sourceID)
{
#ifdef _DUMP_MESSAGES
	std::stringstream _ss;
	_ss << "] " << channelID << "s:" << sourceID << std::endl;
	DUMP_SS(_ss);
#endif

	TFIXMessageEventHandlerMap::iterator it;
	for (it = this->m_fixMessageEventHandlerMap.begin(); it != this->m_fixMessageEventHandlerMap.end(); ++it)
		it->second->onMessageEnd(channelID, sourceID);
}

void CFIXMessageEventHandlerContainer::onField(int fieldID, const std::string& value)
{
	if( value.empty() ) return;

#ifdef _DUMP_MESSAGES
	std::stringstream _ss;
	_ss << fieldID << "=" << value << "\001";
	DUMP_SS(_ss);
#endif

	TFIXMessageEventHandlerMap::iterator it;
	for (it = this->m_fixMessageEventHandlerMap.begin(); it != this->m_fixMessageEventHandlerMap.end(); ++it)
		it->second->onField(fieldID, value);
}

void CFIXMessageEventHandlerContainer::onRepeatingGroupBegin(int fieldID, unsigned int size)
{
#ifdef _DUMP_MESSAGES
	std::stringstream _ss;
	_ss << fieldID << "=" << size << "\001";
	DUMP_SS(_ss);
#endif

	TFIXMessageEventHandlerMap::iterator it;
	for (it = this->m_fixMessageEventHandlerMap.begin(); it != this->m_fixMessageEventHandlerMap.end(); ++it)
		it->second->onRepeatingGroupBegin(fieldID, size);
}

void CFIXMessageEventHandlerContainer::onRepeatingGroupEnd(int fieldID)
{
	TFIXMessageEventHandlerMap::iterator it;
	for (it = this->m_fixMessageEventHandlerMap.begin(); it != this->m_fixMessageEventHandlerMap.end(); ++it)
		it->second->onRepeatingGroupEnd(fieldID);
}

void CFIXMessageEventHandlerContainer::onRepeatingGroupItemBegin(int slot)
{
	TFIXMessageEventHandlerMap::iterator it;
	for (it = this->m_fixMessageEventHandlerMap.begin(); it != this->m_fixMessageEventHandlerMap.end(); ++it)
		it->second->onRepeatingGroupItemBegin(slot);
}

void CFIXMessageEventHandlerContainer::onRepeatingGroupItemEnd(int slot)
{
	TFIXMessageEventHandlerMap::iterator it;
	for (it = this->m_fixMessageEventHandlerMap.begin(); it != this->m_fixMessageEventHandlerMap.end(); ++it)
		it->second->onRepeatingGroupItemEnd(slot);
}

void CFIXMessageEventHandlerContainer::onGroupBegin (int fieldID, int groupNo) 
{
	TFIXMessageEventHandlerMap::iterator it;
	for (it = this->m_fixMessageEventHandlerMap.begin(); it != this->m_fixMessageEventHandlerMap.end(); ++it)
		it->second->onGroupBegin(fieldID, groupNo);
}

void CFIXMessageEventHandlerContainer::onGroupEnd (int fieldID)
{
	TFIXMessageEventHandlerMap::iterator it;
	for (it = this->m_fixMessageEventHandlerMap.begin(); it != this->m_fixMessageEventHandlerMap.end(); ++it)
		it->second->onGroupEnd(fieldID);
}

}