#include <sstream>
#include <boost/format.hpp>
#include "fields/FieldDefinition.h"


#ifndef CME_FAST_UTILS_H_
#define CME_FAST_UTILS_H_

namespace FAST
{

/**
 * This class provides basic functions to various internal classes.
 */
class Utils
{
  public:
    /**
     * @return a string equal to the argument input type.
     */
    template<typename type>
    inline static string to_string(const type& input)
	{
      std::stringstream ss;
      ss << input;
      return ss.str();
    }


    /**
     * @return a zero string of length equal to the argument count.
     */
    template<typename type>
    inline static string zeros(const type& count)
	{
	  string format = boost::str(boost::format::basic_format<char>("%d") %  count);
      string buffer;
	  buffer.reserve(32);
	  buffer.append(boost::str(boost::format::basic_format<char>("%|1$0" + format + "d|") % 0));
      return buffer;
    }

    
    /**
     * @return a hex string for the argument input.
     */
    template<typename type>
    inline static string to_hex(const type& input)
	{
      return boost::str(boost::format::basic_format<char>("%x") %  input);
    }
    

    /**
     * Convert long values to fit into 32 bits of data
     *  
     * @param value the long value
     * @return 32 bit representation of the value inside a 64 bit field.
     */
    static int64_t convertLongUInt(const int64_t& value) {
      const uint64_t unsigned_value = value << 32;
      return unsigned_value >> 32;
    }

    
    /**
     * Is the field always present or will it's presence be defined
     * within a presence map
     * 
     * @param fieldOperator the FieldDefinition.OPERATOR
     * @return is this field always present
     */
    static bool mandatory_field(const int& field_operator, const bool& optional) {
      switch(field_operator) {
        case FieldDefinition::OPERATOR_DELTA:
        case FieldDefinition::OPERATOR_DEFAULT:
          return true;
        case FieldDefinition::OPERATOR_CONSTANT:
          return !optional;
        default:
          return false;
      }
    }
    
  private:
    Utils() {}
    virtual ~Utils() {}
};

}

#endif /*CME_FAST_UTILS_H_*/
