#include <assert.h>
#include <iostream>


#ifndef CME_FAST_CVALUE_H_
#define CME_FAST_CVALUE_H_

#include "CData.h"

using std::ostream;

namespace FAST
{

/**
 * This class provides a generic way to set and get values of any type.
 * Currently it is used for numeric and string types only.
 */
template <typename value_type>
class CValue : public CData {
  public:
    // Note: This is typedefed publicly for easy reference in other classes:
    typedef value_type type;
    
  protected:
    type _value;
    
    CValue() {
      // nothing to do!
    }

  public:
    virtual ~CValue() {
      // nothing to do!
    }
    

    /**
     * @return the value of this CValue or 0 if this CValue is null.
     */
    virtual type get_value() const {
      return _null ? 0 : _value;
    }

    
    /**
     * Sets the internal value to the argument value.  After this method
     * has been called this CValue is no longer null.
     *
     * @param value the value to set this CValue to.
     */
    virtual void set_value(const type& value) {
      _null = false;
      _value = value;
    }


    /**
     * Sets the internal state of this CValue to null.
     */
    virtual void reset() {
      _null = true;
    }


    virtual ostream& insert(ostream& out) const {
      out << "[null=" << (_null ? "true" : "false") << ",value=" << _value << "]";
      return out;
    }

  private:
    /**
     * Disabled.
     */
    CValue(const CValue&) throw();
};

}

#endif /*CME_FAST_CVALUE_H_*/
