#include "stdafx.h"
#include <boost/shared_array.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/filesystem/operations.hpp>
#include <set>
#include <vector>
#include <map>
#include <FASTDecoder/FIXMessageEventHandler.h>
#include <iostream>
#include <fstream>
#include <stdio.h>

class TBuffer
{
public:
	TBuffer(void) : m_dataSize(0) {}
	TBuffer(boost::shared_array<unsigned char> data, size_t dataSize) : m_data(data), m_dataSize(dataSize) {}
	void update(const unsigned char* pCurrentBuffer, size_t currentBufferSize);
	void clear();
	boost::shared_array<unsigned char> getData() const { return m_data; }
	size_t getDataSize() const { return m_dataSize; }
	
private:
	boost::shared_array<unsigned char> m_data;
	std::size_t m_dataSize;
};

class CReceivedBuffer
{
public:
	CReceivedBuffer(unsigned short usNoChunks);
	size_t update(unsigned short usCurrentChunk, const unsigned char* pCurrentBuffer, std::size_t currentBufferSize);
	size_t getRemainingChunk() const;
	boost::shared_ptr<TBuffer> getBufferPtr() const;

private:
	std::set<unsigned short> m_usRemainingChunkSet;
	std::vector<TBuffer> m_bufferVector;
};

class CDataJob
{
public:
	CDataJob(boost::shared_array<unsigned char> buffer, std::size_t bufferSize)
		: m_bufferSize(bufferSize)
	{
		m_buffer.reset(new unsigned char[bufferSize]);
		memcpy_s(m_buffer.get(), m_bufferSize, buffer.get(), bufferSize);
	}

	const unsigned char* getBuffer() const { return m_buffer.get(); }
	std::size_t getBufferSize() const { return m_bufferSize; }

private:
	boost::shared_array<unsigned char> m_buffer;
	size_t m_bufferSize;
};

typedef boost::shared_ptr<CDataJob> TDataJobPtr;
typedef boost::shared_ptr<CReceivedBuffer> TRecvBufferPtr;
typedef std::map<unsigned long, TRecvBufferPtr> TMsgSeqNum2RecvBufferPtrMap;


class CMarketDataMessageEventHandler :
	public FAST::IFIXMessageEventHandler
{
public:
	CMarketDataMessageEventHandler();
	/* virtual */ ~CMarketDataMessageEventHandler(void);

	// Event callbacks
	/* virtual */ void onMessageBegin(int channelID, unsigned int sourceID);
	/* virtual */ void onMessageEnd(int channelID, unsigned int sourceID);
	/* virtual */ void onField(int fieldID, const std::string& value);
	/* virtual */ void onRepeatingGroupBegin(int fieldID, unsigned int size);
	/* virtual */ void onRepeatingGroupEnd(int fieldID);
	/* virtual */ void onRepeatingGroupItemBegin(int slot);
	/* virtual */ void onRepeatingGroupItemEnd(int slot);
	/* virtual */ void onGroupBegin(int fieldID, int groupNo);
	/* virtual */ void onGroupEnd(int fieldID);

private:

};
