#include <map>
#include <set>
#include <string>
#include <vector>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMException.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>

#include <boost/noncopyable.hpp>
#include "Exceptions.h"

#ifdef _WIN32
#include <windows.h>
#endif

#ifndef CME_FAST_ELEMENT_H_
#define CME_FAST_ELEMENT_H_

using std::map;
using std::set;
using std::string;
using std::vector;

namespace FAST
{

/**
 * This class serves as a wrapper for the information contained in an XML
 * element.  There is minimal implementation-specific dependencies, so you
 * are free to implement your XML parsing with your library of choice.
 * Just as long as you populate the Element objects using the values.
 */
class Element : private boost::noncopyable
{
  private:
    static long g_ninstances;
    
  public:
    /**
     * An Element iterator.
     */
    typedef vector<Element *>::iterator Iterator;

    /**
     * @return the root element of the XML file argument.
     */
    static Element* get_root(const string& template_filename, bool validate = false) throw(CXMLParserException);
	static Element* get_root_from_buffer(const XMLByte* const buffer, XMLSize_t bufferSz, bool validate);

  private:
    Element* parent;
    string name;
    vector<Element *> children;
    map<string, string> attributes;

	Element(xercesc::DOMElement* element, Element* parent);
    
  public:
    /**
     * Deletes all allocated memory.
     */
    virtual ~Element();

    /**
     * @return the name of this Element.
     */
    string get_name() const { return name; }

    /**
     * @return an iterator pointed at the first element.
     */
    Iterator children_begin() { return children.begin(); }


    /**
     * @return an iterator pointed <i>after</i> the last element.
     */
    Iterator children_end() { return children.end(); }
    
    /**
     * @param name the name of the argument to access.
     * @return the value of the attribute matching the argument.
     */
    string attribute(const string& name) {
      return attributes[name];
    }
};

}

#endif /*CME_FAST_ELEMENT_H_*/
