#pragma once

#ifndef CME_FAST_EVENTDECODER_H_
#define CME_FAST_EVENTDECODER_H_

#include <boost/noncopyable.hpp>
#include "FIXMessageEventHandler.h"
#include "TransferEncoder.h"

namespace FAST
{

/*
	Event based FAST decoder based on SAX principles (event based parsing)
 */

class CEventDecoder :
	public boost::noncopyable
{
public:
    static const string VERSION;
    static string get_version();

	CEventDecoder(void);
	CEventDecoder(const string& template_file);
	CEventDecoder(const unsigned char* const buffer, unsigned long bufferSz);
	virtual ~CEventDecoder(void);

	// Register/Unregister
	bool registerEventHandler(IFIXMessageEventHandlerPtr fixMessageEventHandlerPtr);
	bool unregisterEventHandler(IFIXMessageEventHandlerPtr fixMessageEventHandlerPtr);

	// Start de decoding process
	void decode(const char* msg, int bufferSize);
	void decode(int channelID, unsigned int sourceID, const char* msg, int bufferSize);

protected:
	TemplateBuilder m_templateBuilder;
	TransferEncoder m_transferEncoder;

	Codec* m_codec;
	TemplateBuilder::TemplateMap m_templates;
	CFIXMessageEventHandlerContainer m_eventHandlerContainer;

};

typedef boost::shared_ptr<CEventDecoder> TEventDecoderPtr;

}

#endif	// CME_FAST_EVENTDECODER_H_