#include "stdafx.h"
#include "FASTDecoder.h"
#include "SampleDecoder.h"

#include <fstream>
#include <iomanip>
#include <iostream>
#include <boost/scoped_array.hpp>

using std::fstream;
using std::ifstream;
using std::ios;
using std::string;

namespace FAST
{

const string SampleDecoder::VERSION = "1.0";


/**
 * @return a string describing the version number.
 */
string SampleDecoder::get_version() {
  stringstream ss;
  ss << "CME Group C++ FixFAST Decoder v" << VERSION;
  return ss.str();
}


/**
 * Creates a SampleDecoder using the argument template file.
 */
SampleDecoder::SampleDecoder(const string& template_file)
{
  codec = m_templateBuilder.create_templates(template_file, templates, &m_transferEncoder);
}


/**
 * Parse the binary file and print out a byte array. This should call the decoder function and pass it the
 * corresponding byte array.
 */
void SampleDecoder::decoderit(const string& data_file)
{
  fstream file(data_file.c_str(), ios::binary | ios::in | ios::ate);
  if (!file || !file.is_open()) {
    std::cerr << "Failed to read data file.  Exiting..." << endl;
    exit(1);
  }

  // Read in the data file into memory
  const ifstream::pos_type size = file.tellg();
  boost::scoped_array<CBytes::byte> buffer(new CBytes::byte[size]);
  file.seekg(0, ios::beg);
  file.read((char *) buffer.get(), size);
  file.close();

  if (LOGME) {
    printf("\nMessage HEX:\n");
    for (int i = 0; i < size; ++i) printf("%02X ",buffer.get()[i]);
    printf("\n\n");
  }

  m_transferEncoder.set_input_stream(buffer.get(), size);
  m_transferEncoder.reset_input_pointer();

  if (LOGME) cout << get_version() << endl;
  cout << decode() << endl;
}

void SampleDecoder::decode_message(const char* msg, int bufferSize)
{
	const CBytes::byte* buffer = (const CBytes::byte *)msg;

	if (LOGME)
	{
		printf("\nMessage HEX:\n");
		for (int i = 0; i < bufferSize; ++i) printf("%02X ",buffer[i]);
		printf("\n\n");
	}

	m_transferEncoder.set_input_stream(buffer, bufferSize);
	m_transferEncoder.reset_input_pointer();

	if (LOGME) cout << get_version() << endl;
	cout << decode() << endl;

	m_transferEncoder.reset_input_pointer();
}

std::string SampleDecoder::decode_message_to_string(const char* msg, int bufferSize)
{
	const CBytes::byte* buffer = (CBytes::byte *)msg;

	if (LOGME)
	{
		printf("\nMessage HEX:\n");
		for (int i = 0; i < bufferSize; ++i) printf("%02X ",buffer[i]);
		printf("\n\n");
	}

	m_transferEncoder.set_input_stream(buffer, bufferSize);
	m_transferEncoder.reset_input_pointer();

	if (LOGME) cout << get_version() << endl;

	std::string decodedMessage(decode());
	return decodedMessage;
}

}