#pragma once

#ifndef UMDF_HEADER_SIZE
//UMDF_HEADER_SIZE = 4B SeqNum - 2B no chunks - 2B current chunk - 2B MsgLen = 10B
#define UMDF_HEADER_SIZE 10
#endif

#include <boost/timer.hpp>

namespace BmfMessagingPlugin
{

	class ChunkData {
	public:
		typedef boost::shared_array<u_char> DataArray;
	private:
		DataArray _data;
		size_t _dataSize;
	public:
		const DataArray &getData() {
			return _data;
		}
		size_t getDataSize() {
			return _dataSize;
		}
		bool update(const unsigned char *data, int size) {
			_data.reset(new u_char[size]);
			memcpy(_data.get(), data, size	);
			_dataSize = (size_t)size;
			return true;
		}
	};

	class ChunkHandler {
	private:
		std::vector<ChunkData> _vecData;
		std::vector<bool> _vecFilled;
		size_t _dataSize;
		
	public:
		boost::timer ChunkTimer;

		ChunkHandler(int noChunks) {
			_vecData.reserve(noChunks);
			_vecFilled.reserve(noChunks);
			for(int i=0; i < noChunks; i++) {
				_vecData.push_back(ChunkData());
				_vecFilled.push_back(false);
			}
			_dataSize = 0;
			for(std::vector<bool>::iterator it=_vecFilled.begin(); it != _vecFilled.end(); it++) {
				*it = false;
			}
		}

		void clear()
		{
			_vecData.clear();
			_vecFilled.clear();
		}

		bool update(int curChunk, const unsigned char *data, int size) {
			if( ((int)_vecData.size()) < curChunk ) {
				//ERRO!
				ASSERT( ((int)_vecData.size()) < curChunk);
				return false;
			}
			_dataSize = 0;
			int pos = curChunk-1;
			ChunkData &chunk = _vecData[pos];
			bool ret;
			if( curChunk == 1 ) {
				ret = chunk.update(data, size);
			} else {
				//Let's take header off.
				ret = chunk.update(data+ UMDF_HEADER_SIZE, size- UMDF_HEADER_SIZE);
			}
			if( ret ) {
				_vecFilled[pos] = true;
			}
			return ret;
		}	

		bool hasFinished() {
			for(std::vector<bool>::iterator it=_vecFilled.begin(); it != _vecFilled.end(); it++) {
				if( (*it) == false ) {
					return false;
				}
			}
			return true;
		}

		size_t getDataSize() {
			//if( _dataSize > 0 ) return _dataSize;
			size_t size=0;
			for(std::vector<ChunkData>::iterator it=_vecData.begin(); it != _vecData.end(); it++) {
				ChunkData &c = *it;
				size += c.getDataSize();
			}
			_dataSize = size;
			return _dataSize;
		}

		void _setInt16(char *buffer, int value) {
			short v = (short)value;
			const char *c = (const char *)&v;
			buffer[0] = c[1];
			buffer[1] = c[0];
		}

		boost::shared_array<char> getData() {
			boost::shared_array<char> ret;
			int totalSize = this->getDataSize();
			ret.reset( new char[ totalSize ] );
			
			int offset = 0;
			for(std::vector<ChunkData>::iterator it=_vecData.begin(); it != _vecData.end(); it++) {
				ChunkData &c = *it;
				int size = c.getDataSize();
				char *p = ret.get();
				p += offset;
				memcpy(p, c.getData().get(), size);
				offset += size;
			}

			char *p = ret.get();
			_setInt16(p+4, 1);
			_setInt16(p+6, 1);
			_setInt16(p+8, totalSize - UMDF_HEADER_SIZE); //we're decreasing header size because this size reflects only fast msg size.

			return ret;
		}

	};

	typedef boost::shared_ptr<ChunkHandler> ChunkHandlerPtr;
	typedef std::map<int, ChunkHandlerPtr> MsgChunkMap;
	typedef std::pair<int, ChunkHandlerPtr> MsgChunkMapItem;

}