#include "StdAfx.h"
#include <Fields/FieldDefinition.h>
#include "MarketDataMessageEventHandler.h"

#include <iostream>
#include <sstream>

namespace BmfMessagingPlugin
{
	CMarketDataMessageEventHandler::CMarketDataMessageEventHandler(TFIXMessageProcessDispatcherWeakPtr wpMessageProcessDispatcher)
		: m_wpMessageProcessDispatcher(wpMessageProcessDispatcher)
	{
		ASSERT(wpMessageProcessDispatcher.lock());
	}

	CMarketDataMessageEventHandler::~CMarketDataMessageEventHandler(void)
	{
	}

	// --------------------------------------------------------------------------------
	//
	// Event callbacks
	//
	// --------------------------------------------------------------------------------
	void CMarketDataMessageEventHandler::onMessageBegin(int channelID, unsigned int sourceID)
	{
		this->m_channelID = channelID;
		this->m_sourceID = sourceID;

		this->m_rawMessage.str("");
		this->m_events.str("");
		
		m_rawMessage << _T("Ch:") << channelID << _T(" s:") << sourceID << _T(": ");
		m_events << "onMessageBegin " << channelID << ", " << sourceID << " - ";
	}


	void CMarketDataMessageEventHandler::onMessageEnd(int channelID, unsigned int sourceID)
	{
		m_events << "onMessageEnd " << channelID << ", " << sourceID << "|";
		//GW_INFO(m_events.str());

		TFIXMessageProcessDispatcherPtr spMessageProcessDispatcher = this->m_wpMessageProcessDispatcher.lock();
		spMessageProcessDispatcher->dispatchMessage(channelID, sourceID, m_rawMessage.str());
	}


	void CMarketDataMessageEventHandler::onField(int fieldID, const std::string& value)
	{
		m_events << "onField " << fieldID << ", " << value << " - ";
		m_rawMessage << fieldID << _T("=") << value << FAST::FieldDefinition::DELIMITER;
	}


	void CMarketDataMessageEventHandler::onRepeatingGroupBegin(int fieldID, unsigned int size)
	{
		m_events << "onRepeatingGroupBegin " << fieldID << ", " << size << " - ";
		m_rawMessage << fieldID << _T("=") << size << FAST::FieldDefinition::DELIMITER;
	}


	void CMarketDataMessageEventHandler::onRepeatingGroupEnd(int fieldID)
	{
		m_events << "onRepeatingGroupEnd " << fieldID << " - ";
	}


	void CMarketDataMessageEventHandler::onRepeatingGroupItemBegin(int slot)
	{
		//m_events << "onRepeatingGroupItemBegin " << slot << " - ";
	}


	void CMarketDataMessageEventHandler::onRepeatingGroupItemEnd(int slot)
	{
		//m_events << "onRepeatingGroupItemEnd " << slot << " - ";
	}


	void CMarketDataMessageEventHandler::onGroupBegin(int fieldID, int groupNo)
	{
		m_events << "onGroupBegin " << fieldID << ", " << groupNo << " - ";
	}


	void CMarketDataMessageEventHandler::onGroupEnd(int fieldID)
	{
		m_events << "onGroupEnd " << fieldID << " - ";
	}

}