#include "StdAfx.h"
#include "MulticastReceiver.h"
#include "Base64.h"

#if( defined(_DUMP_MESSAGES) )
//#define READ_DUMP_FILE
#endif

extern std::string _dumpMessagesPath;
extern bool _dumpMessages;
extern std::string _readDumpPath;

namespace BmfMessagingPlugin
{

	inline int _getInt16(const char *buffer)
	{
		return (short) (((buffer[0] & 0xFF) << 8) | (buffer[1] & 0xFF));
	}

	inline int _getInt32(const char *buffer)
	{
		return ((buffer[0] & 0xFF) << 24) | ((buffer[1] & 0xFF) << 16) | 
			   ((buffer[2] & 0xFF) <<  8) | (buffer[3] & 0xFF);
	}

	size_t _getFastSize(const char *buffer)
	{
		return _getInt16(&buffer[8]);
	}

	// --------------------------------------------------
	// Worker implementation
	// --------------------------------------------------
	void CMulticastReceiver::processJob(DataJobPtr dataJob)
	{
		ASSERT(dataJob.get() != NULL);

		if (dataJob == NULL)
		{
			GW_ERROR_FORMAT(_T("%s(line %d) - %s: Job is invalid!"), __FILE__ % __LINE__ % __FUNCTION__);
			return;
		}

		try
		{
			//GW_DEBUG_FORMAT(_T("dados FAST [%d] - time: %d"), dataJob->getBufferSize() % dataJob->getTime());

			//TODO: Adicionar suporte para mais de uma mensagem por datagram.
			//TODO: Adicionar timeout para a mensagem quebrada
			//TODO: Verificar enfileiramento de mensagem para ordenar.

			//Decode FAST message
			const char *msg = dataJob->getBuffer();
			int seqNum = dataJob->getFixSeqNum();
			//int seqNum = _getInt32(msg);
			//int noChunks = _getInt16(&msg[4]);
			//int curChunk = _getInt16(&msg[6]);
			//int msgSize = _getInt16(&msg[8]);

			STRING code;
			
			// Checkout decoder from pool, decode message and checkin decoder to pool.
			UINT_PTR jobID = reinterpret_cast<UINT_PTR>(dataJob.get());
			//GW_DEBUG_FORMAT(_T("====> %s(%d): Job [#%u] - SRC=[%u] SIZE=[%u] SEQNUM=[%u]."), 
			//	__FUNCTION__ % dataJob->getChannelID() % jobID % dataJob->getSourceID() % dataJob->getBufferSize() % seqNum);
			FAST::TEventDecoderPtr decoder = dataJob->getEventDecoder();

			m_decoder->decode(dataJob->getChannelID(), dataJob->getSourceID(), msg + UMDF_HEADER_SIZE, dataJob->getFastSize());
			
			/*
			GW_DEBUG_FORMAT(_T("<==== %s(%d): Job [#%u] - processed for source=[%u]."), 
				__FUNCTION__ % dataJob->getChannelID() % jobID % dataJob->getSourceID());
			*/
		} catch (std::exception &e) {
			GW_ERROR_FORMAT("%s(%d): Error - source [%d] - %s", __FUNCTION__ % dataJob->getChannelID() % dataJob->getSourceID() % e.what() );
		} catch (...) {
			GW_ERROR_FORMAT("%s(%d): Unknonw Error - source [%d] - ", __FUNCTION__ % dataJob->getChannelID() % dataJob->getSourceID());
		}

	}

#if( defined(_DUMP_MESSAGES) )
	class DumpReader {
	private:
		std::string _fName;
		int _sourceID;
		int _channelID;
		CMulticastReceiver *_recv;
	public:
		DumpReader(const std::string &path, int channelID, int sourceID, CMulticastReceiver *recv)
		{
			//if( sourceID == CSourceConfiguration::Snapshot ) {
			_sourceID = sourceID;
			_channelID = channelID;
			_recv = recv;
			char fbuf[_MAX_PATH];
			sprintf(fbuf, "%s\\%d_fast_%d.log", path.c_str(), channelID, sourceID);
			_fName = fbuf;
		}
		void exec(int x) {
			FILE *f = fopen(_fName.c_str(), "rb");
			if( f == NULL ) return;

			char msg[4096];
			while( !feof(f) ) {

				/*
				__int64 time;
				fread(&time, sizeof(__int64), 1, f);
				*/

				fread(msg, 1, UMDF_HEADER_SIZE, f);
				int seqNum = _getInt32(msg);
				int noChunks = _getInt16(&msg[4]);
				int curChunk = _getInt16(&msg[6]);
				int msgSize = _getInt16(&msg[8]);
				size_t readSize = fread(&msg[UMDF_HEADER_SIZE], 1, msgSize, f);
				
				if( readSize != msgSize ) {
					//Error!
					continue;
				}
				boost::shared_array<char> msgData;
				msgData.reset( new char[msgSize + UMDF_HEADER_SIZE + 1] );
				memcpy(msgData.get(), msg, msgSize + UMDF_HEADER_SIZE);
				CMulticastReceiver::DataJobPtr job( new DataJob(_recv->getChannelID(), _recv->getSourceID(), msgData, msgSize+UMDF_HEADER_SIZE, _recv->getEventDecoder()) );
				_recv->handleMessage( job );
				
				Sleep(1);
			}
			fclose(f);

		}
	};
#endif

	// --------------------------------------------------
	// MulticastReceiver implementation
	// --------------------------------------------------

	// Constructor
	// takes ownership of io_service_ptr

	/*
		const boost::asio::ip::address& listenAddress, 
		const boost::asio::ip::address& multicastAddress, 
		short multicastPort, 
		int recvBufferSize, 
	*/
	CMulticastReceiver::CMulticastReceiver(
		int channelID, 
		unsigned int sourceID, 
		TIOServicePtr ioServicePtr, 
		TSourceConfigurationPtr sourceConfigurationPtr, 
		std::size_t noWorkerThreads, 
		FAST::TEventDecoderPtr decoder):
		m_channelID(channelID),
		m_sourceID(sourceID), 
		m_ioServiceWeakPtr(ioServicePtr), 
		m_socket(*ioServicePtr), 
		m_decoder(decoder), 
		m_recvBufferSize(sourceConfigurationPtr->getReceiveBufferSize()),
		m_multicastPort(sourceConfigurationPtr->getRemotePort()), 
		m_bIsRunning(false),
		m_waitForFirst(false),
		_fDump(NULL),
		m_processingThreadPool(1)
	{
		GW_INFO_FORMAT("%s - CHNL=%03d|SRC=%u: ReceiveBufferSize=[%u]", __FUNCTION__ % this->m_channelID % this->m_sourceID % this->m_recvBufferSize)
		this->m_data.reset(new char[this->m_recvBufferSize]);
		boost::asio::ip::address listenAddress = boost::asio::ip::address::from_string(sourceConfigurationPtr->getLocalAddr().c_str());
		boost::asio::ip::address multicastAddress = boost::asio::ip::address::from_string(sourceConfigurationPtr->getRemoteAddr().c_str());

		// Create Thread Pool
		//this->m_processingThreadPool.start(1, noWorkerThreads);

		m_listenAddress_v4 = boost::asio::ip::address_v4::from_string(sourceConfigurationPtr->getLocalAddr().c_str());		
		m_multicastAddress_v4 = boost::asio::ip::address_v4::from_string(sourceConfigurationPtr->getRemoteAddr().c_str());		
		// Create the socket so that multiple may be bound to the same address.
		boost::asio::ip::udp::endpoint listenEndpoint(m_listenAddress_v4, this->m_multicastPort);
		this->m_socket.open(listenEndpoint.protocol());
		this->m_socket.set_option(boost::asio::ip::udp::socket::reuse_address(true));
		this->m_socket.set_option(boost::asio::ip::udp::socket::receive_buffer_size(this->m_recvBufferSize));
		this->m_socket.bind(listenEndpoint);

		// Join the multicast group.
		this->m_socket.set_option(boost::asio::ip::multicast::join_group(m_multicastAddress_v4,m_listenAddress_v4));

		if( !_readDumpPath.empty() ) {
			boost::thread threadDumpReader(boost::bind(&DumpReader::exec, new DumpReader(_readDumpPath, channelID, sourceID, this), 1));
		} else {
			this->m_socket.async_receive_from(
				boost::asio::buffer(this->m_data.get(), this->m_recvBufferSize), this->m_senderEndpoint,
				boost::bind(&CMulticastReceiver::handleReceiveFrom, this,
				boost::asio::placeholders::error,
				boost::asio::placeholders::bytes_transferred));
		}
	}

	CMulticastReceiver::~CMulticastReceiver(void)
	{
		if( _fDump != NULL ) fclose(_fDump);
	}

	void CMulticastReceiver::handleMessage(DataJobPtr dataJob)
	{

		try
		{
			const char *msg = dataJob->getBuffer();
			int seqNum = _getInt32(msg);
			int noChunks = _getInt16(&msg[4]);
			int curChunk = _getInt16(&msg[6]);
			int msgSize = _getInt16(&msg[8]);

			dataJob->setFastSize(msgSize);
			dataJob->setFixSeqNum(seqNum);

			if (dataJob->getSourceID() == 3)
			{
				GW_DEBUG_FORMAT("%s - CHNL=%03d|SRC=%u: seqnum=[%d] noChunks=[%d] curChunk=[%d] msgSize=[%d]",
					__FUNCTION__ % dataJob->getChannelID() % dataJob->getSourceID() % seqNum % noChunks % curChunk % msgSize);
			}

			//#if( defined(_DUMP_MESSAGES) )
			if( _dumpMessages && _dumpMessagesPath.size() > 0 )
			{
				if( _fDump == NULL ) {
					char fbuf[_MAX_PATH];
					sprintf(fbuf, "%s\\%d_fast_%d.log", _dumpMessagesPath.c_str(), dataJob->getChannelID(), dataJob->getSourceID());
					_fDump = fopen(fbuf, "ab+");
				}
				if( _fDump != NULL ) {
					LARGE_INTEGER ticksPerSecond;
					LARGE_INTEGER ticks;
			        ::QueryPerformanceFrequency (&ticksPerSecond);
		            ::QueryPerformanceCounter (&ticks);

					__int64 time = ticks.QuadPart / ticksPerSecond.QuadPart;
					
					//__int64 time = _ts.getTimestamp();
					fwrite(&time, sizeof(__int64), 1, _fDump);
					fwrite(msg, 1, dataJob->getBufferSize(), _fDump);
					fflush(_fDump);
				}
			}
			//#endif

			if( m_waitForFirst ) {
				if( seqNum == 1 )
					m_waitForFirst = false;
				else
				{
					if (dataJob != NULL)
					{
						dataJob.reset();
					}
					return;
				}
			}

			bool finished = false;

			if( noChunks == 1 ) {
				// Processing received buffer
				finished=true;
			} else {
				boost::mutex::scoped_lock locker(m_multicastLock);

				ChunkHandlerPtr ch;
				MsgChunkMap::iterator it = _mapMessages.find(seqNum);
				if( it == _mapMessages.end() ) {
					ch.reset( new ChunkHandler(noChunks) );
					_mapMessages.insert( MsgChunkMapItem(seqNum, ch) );
				} else {
					ch = it->second;
				}
				ch->update(curChunk, (const unsigned char *)dataJob->getBuffer(), dataJob->getBufferSize());
				if( ch->hasFinished() ) {
					finished = true;

					DataJobPtr ptr (new DataJob( dataJob->getChannelID(), dataJob->getSourceID(), ch->getData(), ch->getDataSize(), dataJob->getEventDecoder() ));
					ptr->setFastSize(ch->getDataSize() - UMDF_HEADER_SIZE);
					ptr->setFixSeqNum(seqNum);
					ptr->setTime( dataJob->getTime() );

					dataJob.reset();
					dataJob = ptr;

					_mapMessages.erase( seqNum );
				}
				//GW_DEBUG_FORMAT( _T("Received message with more than one chunk - seqNum=[%u]:curChunk=[%u]:noChunks=[%u]:source=[%u] - finished=%d"), seqNum % curChunk % noChunks % m_sourceID % finished);

			}

			if( finished ) {
				// Processing received buffer
				//GW_DEBUG_FORMAT(_T("%s(%d): Starting job [#%u] for processing received buffer for source=[%u]: size=[%u]."), 
				//	__FUNCTION__ % this->m_channelID % reinterpret_cast<UINT_PTR>(pJobDesc) % this->m_sourceID % dataJob->getBufferSize());
				if (dataJob->getSourceID() == 3)
				{
					GW_INFO_FORMAT(_T("%s - CHNL=%03d|SRC=%u: received [MsgSeqNum=%d]"), __FUNCTION__ % dataJob->getChannelID() % dataJob->getSourceID() % dataJob->getFixSeqNum());
				}
				//this->m_processingThreadPool.processJob(pJobDesc, m_dataWorkerPtr.get());
				m_processingThreadPool.schedule(boost::bind(&CMulticastReceiver::processJob, this, dataJob));
			}
		}
		catch (...)
		{
			GW_ERROR_FORMAT("%s(%d): Unknonw Error - source [%d] - ", __FUNCTION__ % dataJob->getChannelID() % dataJob->getSourceID());
		}

	}

	void CMulticastReceiver::handleReceiveFrom(const boost::system::error_code& errorCode, size_t bytesReceived)
	{
		if (errorCode)
		{
			GW_WARNING_FORMAT(_T("%s(line %d) - %s(%d): Error on handle received message: source=[%u], category=[%s], code=[%d]"), 
				__FILE__ % __LINE__ % __FUNCTION__ % this->m_channelID % this->m_sourceID % errorCode.category().name() % errorCode.value());
		}
		else
		{
			const char *msg = this->m_data.get();

			if (msg == nullptr)
			{
				GW_TRACE_FORMAT("%s - CHNL=%03d|SRC=%u: NULL received", __FUNCTION__ % this->m_channelID % this->m_sourceID);
				this->m_data.reset(new char[this->m_recvBufferSize]);
				memset(this->m_data.get(), 0x00, this->m_recvBufferSize);
			}
			else
			{
				size_t total = 0;
				int msgNum = 0;

				while (total < bytesReceived)
				{
					msgNum++;

					int seqNum = _getInt32(msg);
					int noChunks = _getInt16(&msg[4]);
					int curChunk = _getInt16(&msg[6]);
					size_t size = _getFastSize(msg) + UMDF_HEADER_SIZE;

					if (size == 0 || curChunk == 0 || noChunks == 0 || size > bytesReceived || curChunk > noChunks)
					{
						GW_ERROR_FORMAT("%s - CHNL=%03d|SRC=%u: Format Error - SeqNum=[%d] CurChunk=[%d] NoChunks=[%d] Size=[%d/%d]",
							__FUNCTION__ % this->m_channelID % this->m_sourceID % seqNum % curChunk % noChunks % size % bytesReceived);
						memset(this->m_data.get(), 0x00, this->m_recvBufferSize);
						break;
					}
					else if (size < bytesReceived)
					{
						//GW_TRACE_FORMAT("%s - CHNL=%03d|SRC=%u: Multiple Messages %d - SeqNum %d. size: %d/%d",
						//	__FUNCTION__ % this->m_channelID % this->m_sourceID % msgNum % seqNum % size % bytesReceived);
					}
					else
					{
						//GW_TRACE_FORMAT("%s - CHNL=%03d|SRC=%u: Unique Message %d - SeqNum %d. size: %d/%d",
						//	__FUNCTION__ % this->m_channelID % this->m_sourceID % msgNum % seqNum % size % bytesReceived);
					}

					total += size;
					boost::shared_array<char> data(new char[size]);
					memcpy(data.get(), msg, size);

					// Processing received buffer
					DataJobPtr dataJob(new DataJob(this->m_channelID, this->m_sourceID, data, size, this->m_decoder));

					handleMessage(dataJob);

					msg += size;
				}
			}
		}

		this->m_socket.async_receive_from(
			boost::asio::buffer(this->m_data.get(), this->m_recvBufferSize), this->m_senderEndpoint,
			boost::bind(&CMulticastReceiver::handleReceiveFrom, this, boost::asio::placeholders::error, boost::asio::placeholders::bytes_transferred));
	}

	void CMulticastReceiver::runThread(std::size_t threadPoolSize)
	{
		GW_INFO_FORMAT(_T("%s(%d): Starting [%u] I/O Service(s) for source=[%u] ..."), 
			__FUNCTION__ % this->m_channelID % threadPoolSize % this->m_sourceID);
		this->m_bIsRunning = true;

		// Create a pool of threads to run all of the io_services.
		TIOServicePtr ioServicePtr = this->m_ioServiceWeakPtr.lock();
		if (ioServicePtr)
		{
			if( !_readDumpPath.empty() ) {

				boost::thread threadDumpReader(boost::bind(&DumpReader::exec, new DumpReader(_readDumpPath, this->m_channelID, this->m_sourceID, this), 1));

			} else {
				ioServicePtr->reset();
				std::vector<boost::shared_ptr<boost::thread> > threads;
				for (std::size_t i = 0; i < threadPoolSize; ++i)
				{
					boost::shared_ptr<boost::thread> thread(new boost::thread(
						boost::bind(&boost::asio::io_service::run, ioServicePtr.get())));
					threads.push_back(thread);
				}

				// Wait for all threads in the pool to exit.
				for (std::size_t i = 0; i < threads.size(); ++i)
					threads[i]->join();

				this->m_bIsRunning = false;
				GW_INFO_FORMAT(_T("%s(%d): I/O Service(s) stopped for source=[%u]!"), 
					__FUNCTION__ % this->m_channelID % this->m_sourceID);
			}
		}
	}

}