#pragma once

#ifndef __MULTICASTRECEIVER_H__
#define __MULTICASTRECEIVER_H__

#include "MarketDataReceiverConfiguration.h"
#include <boost/weak_ptr.hpp>
#include "IMulticastReceiver.h"
#include "ChunkHandler.h"
#include "NanosTimestamp.hpp"
#include <boost/threadpool.hpp>

namespace BmfMessagingPlugin
{
	typedef boost::shared_ptr<boost::asio::io_service> TIOServicePtr;
	typedef boost::weak_ptr<boost::asio::io_service> TIOServiceWeakPtr;

	class DataJob
	{
	public:
		DataJob(int channelID, unsigned int sourceID, boost::shared_array<char> buffer, size_t bufferSize, FAST::TEventDecoderPtr decoder)
			: m_channelID(channelID), m_sourceID(sourceID), m_bufferSize(bufferSize), m_decoder(decoder), m_dataSize(bufferSize), m_fixSeqNum(0)
		{
			this->m_buffer.reset(new char[bufferSize]);
			memcpy_s(this->m_buffer.get(), this->m_bufferSize, buffer.get(), this->m_bufferSize);
		}

		int getChannelID() const { return this->m_channelID; }
		unsigned int getSourceID() const { return this->m_sourceID; }
		const char* getBuffer() const { return this->m_buffer.get(); }
		size_t getBufferSize() const { return this->m_bufferSize; }
		FAST::TEventDecoderPtr getEventDecoder() const { return this->m_decoder; }

		void setFastSize(int size) { m_dataSize = size; }
		int getFastSize() { return m_dataSize; }
		void setFixSeqNum(int fixSeqNum) { m_fixSeqNum = fixSeqNum; }
		int getFixSeqNum() { return m_fixSeqNum; }
		void setTime(__int64 time) { _time = time; }
		__int64 getTime() { return _time; }

	private:
		int m_channelID;
		unsigned int m_sourceID;
		boost::shared_array<char> m_buffer;
		size_t m_bufferSize;
		FAST::TEventDecoderPtr m_decoder;
		int m_dataSize;
		int m_fixSeqNum;
		__int64 _time;
	};

	class CMulticastReceiver : public IMulticastReceiver, private boost::noncopyable
	{
	public:

		typedef boost::shared_ptr<DataJob> DataJobPtr;

		// Functional constructor
		CMulticastReceiver(
			int channelID, 
			unsigned int sourceID, 
			TIOServicePtr ioServicePtr, 
			TSourceConfigurationPtr sourceConfigurationPtr, 
			std::size_t noWorkerThreads, 
			FAST::TEventDecoderPtr decoder);

		// Destructor
		virtual ~CMulticastReceiver(void);

		// Callback executed when a datagram is received
		void handleReceiveFrom(const boost::system::error_code& errorCode, size_t bytesReceived);

		//handle message chunks
		void CMulticastReceiver::handleMessage(DataJobPtr dataJob);

		// Run method
		void runThread(std::size_t threadPoolSize);

		// Getters
		bool isRunning() const { return this->m_bIsRunning; }

		int getChannelID() { return this->m_channelID; }
		unsigned int getSourceID() { return this->m_sourceID; }
		FAST::TEventDecoderPtr getEventDecoder() { return this->m_decoder; }
		void setWaitForFirstMessageInLoop(bool value) { m_waitForFirst = value; }

		void processJob(DataJobPtr dataJobPtr);

	private:
		int m_channelID;
		unsigned int m_sourceID;
		TIOServiceWeakPtr m_ioServiceWeakPtr;
		boost::asio::ip::udp::socket m_socket;
		boost::asio::ip::udp::endpoint m_senderEndpoint;
		const int m_multicastPort;
		const int m_recvBufferSize;
		boost::shared_array<char> m_data;
		boost::threadpool::pool m_processingThreadPool;
		FAST::TEventDecoderPtr m_decoder;
		//boost::shared_ptr<CDataWorker> m_dataWorkerPtr;
		bool m_waitForFirst;

		bool m_bIsRunning;								// Status of running I/O service

		MsgChunkMap _mapMessages;

		boost::asio::ip::address_v4 m_listenAddress_v4;		
		boost::asio::ip::address_v4 m_multicastAddress_v4;

		boost::mutex m_multicastLock;

		FILE *_fDump;
		//NanosTimestamp _ts;

	};

	typedef boost::shared_ptr<IMulticastReceiver> TMulticastReceiverPtr;

}

#endif	// __MULTICASTRECEIVER_H__