﻿#pragma once
#ifndef __UMDFMARKETDATARECEIVER_H__
#define __UMDFMARKETDATARECEIVER_H__

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the UMDFMARKETDATARECEIVER_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
//  functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.

#include <boost/enable_shared_from_this.hpp>
#include "MulticastReceiver.h"
#include "MarketDataReceiverConfiguration.h"
#include "fixmessageprocessdispatcher.h"

namespace BmfMessagingPlugin
{

	class  CUMDFMarketDataReceiver : 
		public IFIXMessageProcessDispatcher,
		public boost::enable_shared_from_this<CUMDFMarketDataReceiver>
	{
	public:
		CUMDFMarketDataReceiver(TChannelConfigurationPtr channelConfigurationPtr);
		virtual ~CUMDFMarketDataReceiver(void);

		// Sets up threading environment
		void start(HANDLE hStopEvent);

		// Stop threading environment
		void stop(void);

		void dispatchMessage(int channel, int source, const std::string &msg);

	private:
		int m_iFd;											// Socket file descriptor

		unsigned int m_threadId;							// Thread ID
		HANDLE m_hProcessMessageThread;						// Handle of thread that receives datagrams
		HANDLE m_hStopEvent;								// Stop handler (manual reset)

		// I/O Service for instrument/snapshot/incremental
		TIOServicePtr m_instrumentDefinitionServicePtr;
		TIOServicePtr m_snapshotServicePtr;
		TIOServicePtr m_incrementalServicePtr;

		// Multicast receivers for instrument/snapshot/incremental
		TMulticastReceiverPtr m_instrumentDefinitionReceiverPtr;
		TMulticastReceiverPtr m_snapshotReceiverPtr;
		TMulticastReceiverPtr m_incrementalReceiverPtr;

		// Configuration reference
		const TChannelConfigurationPtr m_channelConfigurationPtr;
		int m_channelID;
		__int64 _lastTime;

	};

	typedef boost::shared_ptr<CUMDFMarketDataReceiver> TUMDFMarketDataReceiverPtr;
	typedef std::map<int, TUMDFMarketDataReceiverPtr> TUMDFMarketDataReceiverPtrMap;

}

#endif // __UMDFMARKETDATARECEIVER_H__