#pragma once

#include <string>
#include <hash_map>
#include <boost/shared_ptr.hpp>
#ifdef _DUMP_MESSAGES
#include <sstream>
#endif

#ifndef UINT_PTR
	#if defined(_WIN64)
		typedef unsigned __int64 UINT_PTR;
	#else
		typedef _W64 unsigned int UINT_PTR;
	#endif
#endif

namespace FAST
{

class IFIXMessageEventHandler
{
public:
	// Event callbacks
	virtual void onMessageBegin(int channelID, unsigned int sourceID) = 0;
	virtual void onMessageEnd(int channelID, unsigned int sourceID) = 0;
	virtual void onField(int fieldID, const std::string& value) = 0;
	virtual void onRepeatingGroupBegin(int fieldID, unsigned int size) = 0;
	virtual void onRepeatingGroupEnd(int fieldID) = 0;
	virtual void onRepeatingGroupItemBegin(int slot) = 0;
	virtual void onRepeatingGroupItemEnd(int slot) = 0;
	virtual void onGroupBegin (int fieldID, int groupNo) = 0;
	virtual void onGroupEnd (int fieldID) = 0;
};

typedef boost::shared_ptr<IFIXMessageEventHandler> IFIXMessageEventHandlerPtr;
typedef stdext::hash_map<UINT_PTR, IFIXMessageEventHandlerPtr> TFIXMessageEventHandlerMap;

class CFIXMessageEventHandlerContainer
{
public:
	CFIXMessageEventHandlerContainer(void);
	virtual ~CFIXMessageEventHandlerContainer(void);

	// Register/Unregister
	bool registerEventHandler(IFIXMessageEventHandlerPtr fixMessageEventHandlerPtr);
	bool unregisterEventHandler(IFIXMessageEventHandlerPtr fixMessageEventHandlerPtr);

	// Event callbacks
	void onMessageBegin(int channelID, unsigned int sourceID);
	void onMessageEnd(int channelID, unsigned int sourceID);
	void onField(int fieldID, const std::string& value);
	void onRepeatingGroupBegin(int fieldID, unsigned int size);
	void onRepeatingGroupEnd(int fieldID);
	void onRepeatingGroupItemBegin(int slot);
	void onRepeatingGroupItemEnd(int slot);
	void onGroupBegin (int fieldID, int groupNo);
	void onGroupEnd (int fieldID);
protected:
	TFIXMessageEventHandlerMap m_fixMessageEventHandlerMap;

#ifdef _DUMP_MESSAGES
	int _channelID;
	int _sourceID;
#endif

};

}