#include <cassert>
#include <boost/noncopyable.hpp>

#ifndef CME_FAST_PRESENCE_MAP_H_
#define CME_FAST_PRESENCE_MAP_H_

namespace FAST
{

/**
 * A PresenceMap is a bit field that determines which, if any, of the fields
 * in a template are present in the following message bits.  A PresenceMap 
 * always comes before the fields it describes.
 */
class PresenceMap : private boost::noncopyable
{
  private:
    const int _slots;
    bool* bits;
    int _size;
    
  public:
    PresenceMap(const int slots)
      :_slots(slots), _size(0)
    {
      bits = new bool[slots];
    }

    
    virtual ~PresenceMap() {
      if (bits != 0) delete [] bits;
      bits = 0;
      _size = 0;
    }

    
    bool* get_bits() const {
      return bits;
    }

    
    int get_size() const {
      return _size;
    }

    
    void set_size(const int size) {
      _size = size;
    }
    

    void reset() {
      while (_size > 0) bits[--_size] = false;
    }
    

    bool get_bit(const int slot) const {
      assert(slot < _slots);
      return bits[slot];
    }

    
    void set_bit(const int slot) {
      assert(slot < _slots);
      if (slot >= _size) _size = slot + 1;
      bits[slot] = true;
    }
};

}

#endif /*CME_FAST_PRESENCE_MAP_H_*/
