#include <sstream>
#include <boost/noncopyable.hpp>

#include "values/CValues.h"
#include "TemplateBuilder.h"
#include "Decoder.h"
#include "FLogger.h"

#ifndef CME_FAST_TRANSFER_ENCODER_H_
#define CME_FAST_TRANSFER_ENCODER_H_

namespace FAST
{

class TransferEncoder : private boost::noncopyable
{
  private:
    typedef uint8_t byte;
    
  public:
    static const int 
      PARSE_BUFFER_SIZE = 1420,
      SIGN_MASK_UNSIGNED = 0x00,
      SIGN_MASK_SIGNED = 0x40;
    
  private:
    CBytes* parse_buffer;
    const CBytes::byte* _input;
	CBytes::byte* bytes;
    int input_pointer, input_length;    
    int parse_bytes();
    
  public:
    TransferEncoder();
    virtual ~TransferEncoder();
    
    inline void set_input_stream(const CBytes::byte* input, const int length)
	{
      _input = input;
      input_length = length;
    }
    
    inline void reset_input_pointer()
	{
      input_pointer = 0;
    }
    
    void parse_bytes(CBytes& data, const int size);
//    void parse_string(const bool slot, CString& value);
	void parse_string(const bool slot, CString& value, const bool optional);
    
    void parse_int(const bool slot, CInt32& value, const bool optional);
    void parse_i64(const bool slot, CInt64& value, const bool optional);
    void parse_int(const bool slot, CuInt32& value, const bool optional);
    void parse_u64(const bool slot, CuInt64& value, const bool optional);
    
    template<class type>
    void parse_int(CValue<type>& value, const type& sign_mask, const bool is_optional);
    
    template<class type>
    void parse_int(const bool slot, CValue<type>& value, const bool optional);
    
  private:
    void parse_int(CValue<CInt64::type>& value, const bool optional);
    void parse_int(CValue<CuInt64::type>& value, const bool optional);
    void parse_int(CValue<CInt32::type>& value, const bool optional);
    void parse_int(CValue<CuInt32::type>& value, const bool optional);
};

}

#endif /*CME_FAST_TRANSFER_ENCODER_H_*/
