#include "../FASTDecoder.h"

namespace FAST
{

typedef Codec::Definitions::iterator Iterator;


GroupFieldDefinition::GroupFieldDefinition(int id, int type, int op, int tid, int s, Codec& codec, bool optional)
  :FieldDefinition(id, type, op, tid, s, optional), group_codec(codec)
{
  presence_map = false;

  // check to see if a presence map is required for this group
  Codec::Definitions& definitions = codec.get_definitions();
  for (Iterator i = ++definitions.begin(), e = definitions.end(); i != e; ++i) {
    if (!Utils::mandatory_field((*i)->get_field_operator(), (*i)->is_optional())) {
      presence_map = true;
      break;
    }
  }

  // if the group is optional notify the group length
  if (is_optional()) {
    definitions[0]->set_optional(true);
  }
}


string GroupFieldDefinition::decode(const int slot, Codec& codec) {
  stringstream ss;
  Codec::Definitions& definitions = group_codec.get_definitions();

  U32FieldDefinition* gfd = (U32FieldDefinition *) definitions.at(0);
  gfd->reset_data();
  gfd->set_slot(0); // TODO - make sure this is correct - was set_slot(slot)
  group_length = CuInt32();

  if (Decoder::LOGME) {
    FLogger::get_instance().set_id(Utils::to_string(gfd->get_field_id()));
    FLogger::get_instance().set_field(gfd->get_field_type_name());
    FLogger::get_instance().set_operator(gfd->get_field_operator_name());
    FLogger::get_instance().set_optional(gfd->is_optional() ? "true" : "false");
    FLogger::get_instance().set_pmap("M");

    string currentValue = (codec.get_current_values().is_valid(get_slot())) ?
      Utils::to_string(codec.get_current_values().get_u32(slot)) : "null";
    FLogger::get_instance().set_current(currentValue);
  }
  // We assume that the length field is mandatory, hence presence is assumed below:
  codec.decode(reinterpret_cast<CuInt32 *>(gfd->get_data()), gfd, true);
  group_length = *(reinterpret_cast<CuInt32 *>(gfd->get_data()));

  if (Decoder::LOGME)
  {
    stringstream ss2;
    ss2 << gfd->get_field_id() << "=" << gfd->get_value();
    FLogger::get_instance().set_fix(ss2.str());
    FLogger::get_instance().pretty_print_display();
    FLogger::get_instance().reset();
  }

  if (gfd->get_data()->is_null()) return "";

  ss << gfd->get_field_id() << "=" << group_length.get_value() << DELIMITER;
  for (unsigned int i = 0; i < group_length.get_value(); ++i)
  {
    CBytesPtr pmap;
	if (presence_map)
		pmap = group_codec.get_presence_map_off_stream();

    group_codec.parse_pmap(pmap, presence_map, 1, false);

    if (Decoder::LOGME) {
      FLogger::get_instance().print_buffer();
      FLogger::get_instance().reset_buffer();
    }

    for (int slot = 1, last_slot = definitions.size(); slot < last_slot; ++slot)
      ss << definitions[slot]->decode(slot, group_codec);
  }

  return parsed_value = ss.str();
}

void GroupFieldDefinition::decode(const int slot, Codec& codec, CFIXMessageEventHandlerContainer& eventHandlerContainer)
{
  Codec::Definitions& definitions = group_codec.get_definitions();

  U32FieldDefinition* gfd = (U32FieldDefinition *) definitions.at(0);
  gfd->reset_data();
  gfd->set_slot(0); // TODO - make sure this is correct - was set_slot(slot)
  group_length = CuInt32();

  if (Decoder::LOGME)
  {
    FLogger::get_instance().set_id(Utils::to_string(gfd->get_field_id()));
    FLogger::get_instance().set_field(gfd->get_field_type_name());
    FLogger::get_instance().set_operator(gfd->get_field_operator_name());
    FLogger::get_instance().set_optional(gfd->is_optional() ? "true" : "false");
    FLogger::get_instance().set_pmap("M");

    string currentValue = (codec.get_current_values().is_valid(get_slot())) ?
      Utils::to_string(codec.get_current_values().get_u32(slot)) : "null";
    FLogger::get_instance().set_current(currentValue);
  }

  // We assume that the length field is mandatory, hence presence is assumed below:
  codec.decode(reinterpret_cast<CuInt32 *>(gfd->get_data()), gfd, true);
  group_length = *(reinterpret_cast<CuInt32 *>(gfd->get_data()));

  if (Decoder::LOGME)
  {
    stringstream ss;
    ss << gfd->get_field_id() << "=" << gfd->get_value();
    FLogger::get_instance().set_fix(ss.str());
    FLogger::get_instance().pretty_print_display();
    FLogger::get_instance().reset();
  }

  if (gfd->get_data()->is_null())
	  return;

  int field_id = gfd->get_field_id();
  eventHandlerContainer.onRepeatingGroupBegin(field_id, group_length.get_value());
  for (unsigned int i = 0; i < group_length.get_value(); ++i)
  {
    CBytesPtr pmap;
	if (presence_map)
		pmap = group_codec.get_presence_map_off_stream();

    group_codec.parse_pmap(pmap, presence_map, 1, false);

    if (Decoder::LOGME)
	{
      FLogger::get_instance().print_buffer();
      FLogger::get_instance().reset_buffer();
    }
	eventHandlerContainer.onGroupBegin (field_id, i);
    for (int slot = 1, last_slot = definitions.size(); slot < last_slot; ++slot)
	{
	  eventHandlerContainer.onRepeatingGroupItemBegin(slot);
      definitions[slot]->decode(slot, group_codec, eventHandlerContainer);
	  eventHandlerContainer.onRepeatingGroupItemEnd(slot);
    }
	eventHandlerContainer.onGroupEnd (field_id);
  }
  eventHandlerContainer.onRepeatingGroupEnd(field_id);
}

}