#include "StdAfx.h"
#include "MarketDataReceiverConfiguration.h"
#include <Element.h>

#define	XML_ATTRIBUTE_STARTUPPRIORITY							_T("startupPriority")
#define	XML_ATTRIBUTE_CONFIGNAME    							_T("configName")
#define XML_ATTRIBUTE_CHUNKFASTTIMEOUTSECONDS					_T("chunkFastTimeoutSeconds")
#define CHUNKFASTTIMEOUT_DEFAULT								120
#define	XML_ATTRIBUTE_STARTUPPRIORITY_VALUE_HIGH_PRIORITY		_T("high")
#define	XML_ATTRIBUTE_STARTUPPRIORITY_VALUE_ABOVE_PRIORITY		_T("above")

#define	XML_ATTRIBUTE_APPLOGFILEPATH					_T("appLogFilePath")
#define	XML_ATTRIBUTE_QUICKFIXCONFIGRATIONFILEPATH		_T("quickfixConfigrationFilePath")
#define	XML_ATTRIBUTE_TEMPLATEFILEPATH					_T("templateFilePath")
#define XML_ATTRIBUTE_CAMLCONFIGFILEPATH				_T("camlConfigFilePath")


#define	XML_ATTRIBUTE_SOURCETYPE						_T("type")
#define	XML_ATTRIBUTE_USECAML							_T("useCaml")
#define	XML_ATTRIBUTE_SOURCETYPE_VALUE_INCREMENTAL		_T("incremental")
#define	XML_ATTRIBUTE_SOURCETYPE_VALUE_SNAPSHOT			_T("snapshot")
#define	XML_ATTRIBUTE_SOURCETYPE_VALUE_HISTORICALREPLAY	_T("historicalreplay")
#define	XML_ATTRIBUTE_SOURCETYPE_VALUE_INSTRUMENTREPLAY	_T("instrumentreplay")

#define XML_ATTRIBUTE_LOCALADDR			_T("localAddr")
#define XML_ATTRIBUTE_REMOTEADDR		_T("remoteAddr")
#define XML_ATTRIBUTE_REMOTEPORT		_T("remotePort")
#define XML_ATTRIBUTE_RECEIVEBUFFERSIZE	_T("receiveBufferSize")
#define XML_ATTRIBUTE_USERNAME			_T("username")
#define XML_ATTRIBUTE_PASSWORD			_T("password")
#define XML_ATTRIBUTE_TIMEOUT			_T("timeout")
#define XML_ATTRIBUTE_TRANSPORTID		_T("transportId")
#define XML_ATTRIBUTE_TOPICID			_T("topicId")
#define XML_ATTRIBUTE_RECEIVERID		_T("receiverId")

#define XML_ATTRIBUTE_CHANNEL_NUMBER			_T("number")
#define XML_ATTRIBUTE_CHANNEL_DEFINITION		_T("definition")
#define XML_ATTRIBUTE_CHANNEL_SECURITYEXCHANGE	_T("defaultSecurityExchange")
#define XML_ATTRIBUTE_CHANNEL_SECURITYPREFIX	_T("securityPrefix")
#define XML_ATTRIBUTE_CHANNEL_NOWORKERTHREADS	_T("noWorkerThreads")
#define XML_ATTRIBUTE_CHANNEL_SMALLSCALERETRIES	_T("smallScaleRetries")
#define XML_ATTRIBUTE_CHANNEL_LARGESCALERETRIES	_T("largeScaleRetries")
#define XML_ATTRIBUTE_CHANNEL_RETRYTIMEOUT		_T("retryTimeout")
#define XML_ATTRIBUTE_CHANNEL_MAXNOMSGQUEUE		_T("maxnomsgqueue")



namespace BmfMessagingPlugin
{
	std::string MarketDataReceiverConfiguration::g_configurationFilePath;
	std::string MarketDataReceiverConfiguration::g_pluginChannel;

	CSourceConfiguration::CSourceConfiguration(EnumSourceType sourceType, const std::string& localAddr, const std::string& remoteAddr, int remotePort, int receiveBufferSize, std::size_t noWorkerThreads)
		: m_sourceType(sourceType), m_localAddr(localAddr), m_remoteAddr(remoteAddr), m_remotePort(remotePort), m_receiveBufferSize(receiveBufferSize), m_noWorkerThreads(noWorkerThreads), m_timeout(0)
	{
	}

	CSourceConfiguration::CSourceConfiguration(EnumSourceType sourceType, const std::string& transportId, const std::string& topicId, const std::string& receiverId)
		: m_sourceType(sourceType), m_transportId(transportId), m_topicId(topicId), m_receiverId(receiverId), m_timeout(0)
	{
	}

	CSourceConfiguration::CSourceConfiguration(FAST::Element* pSourceConfigurationElement, std::size_t noWorkerThreads)
		: m_noWorkerThreads(noWorkerThreads), m_timeout(0)
	{
		assert(pSourceConfigurationElement != NULL);

		std::string strSourceType = pSourceConfigurationElement->attribute(XML_ATTRIBUTE_SOURCETYPE);
		std::transform(strSourceType.begin(), strSourceType.end(), strSourceType.begin(), tolower);
		this->m_useCaml = false;

		if (strSourceType.compare(XML_ATTRIBUTE_SOURCETYPE_VALUE_INCREMENTAL) == 0)
		{
			this->m_sourceType = CSourceConfiguration::Incremental;
			if( stricmp(pSourceConfigurationElement->attribute(XML_ATTRIBUTE_USECAML).c_str(), "true") == 0)
				this->m_useCaml = true;
		}
		else if (strSourceType.compare(XML_ATTRIBUTE_SOURCETYPE_VALUE_SNAPSHOT) == 0)
		{
			this->m_sourceType = CSourceConfiguration::Snapshot;
		}
		else if (strSourceType.compare(XML_ATTRIBUTE_SOURCETYPE_VALUE_INSTRUMENTREPLAY) == 0)
		{
			this->m_sourceType = CSourceConfiguration::InstrumentReplay;
		}
		else
		{
			this->m_sourceType = CSourceConfiguration::HistoricalReplay;
			this->m_username = pSourceConfigurationElement->attribute(XML_ATTRIBUTE_USERNAME);
			this->m_password = pSourceConfigurationElement->attribute(XML_ATTRIBUTE_PASSWORD);
			this->m_timeout = ::_ttoi((pSourceConfigurationElement->attribute(XML_ATTRIBUTE_TIMEOUT)).c_str());
		}

		if (this->isUsingCaml())
		{
			this->m_transportId = pSourceConfigurationElement->attribute(XML_ATTRIBUTE_TRANSPORTID);
			this->m_topicId = pSourceConfigurationElement->attribute(XML_ATTRIBUTE_TOPICID);
			this->m_receiverId = pSourceConfigurationElement->attribute(XML_ATTRIBUTE_RECEIVERID);
		}
		else
		{
			this->m_localAddr = pSourceConfigurationElement->attribute(XML_ATTRIBUTE_LOCALADDR);
			this->m_remoteAddr = pSourceConfigurationElement->attribute(XML_ATTRIBUTE_REMOTEADDR);
			this->m_remotePort = ::_ttoi((pSourceConfigurationElement->attribute(XML_ATTRIBUTE_REMOTEPORT)).c_str());
			this->m_receiveBufferSize = ::_ttoi((pSourceConfigurationElement->attribute(XML_ATTRIBUTE_RECEIVEBUFFERSIZE)).c_str());
		}
	
		if ( !pSourceConfigurationElement->attribute(XML_ATTRIBUTE_CHUNKFASTTIMEOUTSECONDS).empty() )
			this->m_chunkFastTimeoutSeconds = ::_ttoi((pSourceConfigurationElement->attribute(XML_ATTRIBUTE_CHUNKFASTTIMEOUTSECONDS)).c_str());
		else
			this->m_chunkFastTimeoutSeconds = CHUNKFASTTIMEOUT_DEFAULT;

	}

	CSourceConfiguration::~CSourceConfiguration()
	{
	}

	CChannelConfiguration::CChannelConfiguration(FAST::Element* pChannelConfigurationElement, 
												 const std::string& templateFilePath,
												 const std::string& camlConfigFilePath)
		: m_smallScaleRetries(10), m_largeScaleRetries(100), m_maxNoMessageQueue(MAX_NOMSG_QUEUE), 
		m_retryTimeout(200), m_templateFilePath(templateFilePath), m_camlConfigFilePath(camlConfigFilePath)
	{
		assert(pChannelConfigurationElement != NULL);
		std::string buffer;

		this->m_number = ::_ttoi((pChannelConfigurationElement->attribute(XML_ATTRIBUTE_CHANNEL_NUMBER)).c_str());
		if (!this->m_number)
			throw CChannelConfigurationException(_T("Invalid channel number"));
		this->m_definition = pChannelConfigurationElement->attribute(XML_ATTRIBUTE_CHANNEL_DEFINITION);		
		this->m_defaultSecurityExchange = pChannelConfigurationElement->attribute(XML_ATTRIBUTE_CHANNEL_SECURITYEXCHANGE);
		this->m_securityPrefix = pChannelConfigurationElement->attribute(XML_ATTRIBUTE_CHANNEL_SECURITYPREFIX);

		std::size_t noWorkerThreads = 2;
		buffer.assign(pChannelConfigurationElement->attribute(XML_ATTRIBUTE_CHANNEL_NOWORKERTHREADS));
		if (buffer.empty() == false)
			noWorkerThreads = ::_ttoi(buffer.c_str());

		buffer.assign(pChannelConfigurationElement->attribute(XML_ATTRIBUTE_CHANNEL_SMALLSCALERETRIES));
		if (buffer.empty() == false)
			this->m_smallScaleRetries = ::_ttoi(buffer.c_str());
		buffer.assign(pChannelConfigurationElement->attribute(XML_ATTRIBUTE_CHANNEL_LARGESCALERETRIES));
		if (buffer.empty() == false)
			this->m_largeScaleRetries = ::_ttoi(buffer.c_str());
		buffer.assign(pChannelConfigurationElement->attribute(XML_ATTRIBUTE_CHANNEL_RETRYTIMEOUT));
		if (buffer.empty() == false)
			this->m_retryTimeout = ::_ttol(buffer.c_str());
		buffer.assign(pChannelConfigurationElement->attribute(XML_ATTRIBUTE_CHANNEL_MAXNOMSGQUEUE));
		if (buffer.empty() == false)
			this->m_maxNoMessageQueue = ::_ttol(buffer.c_str());

		for (FAST::Element::Iterator it = pChannelConfigurationElement->children_begin(); it != pChannelConfigurationElement->children_end(); ++it)
		{
			TSourceConfigurationPtr sourceConfigurationPtr(new CSourceConfiguration(*it, noWorkerThreads));
			this->m_sourceConfigurationPtrMap.insert(TSourceConfigurationPtrMap::value_type(sourceConfigurationPtr->getSourceType(), sourceConfigurationPtr));
		}
	}

	CChannelConfiguration::~CChannelConfiguration(void)
	{
		this->m_sourceConfigurationPtrMap.clear();
	}

	TSourceConfigurationPtr CChannelConfiguration::getSourceConfigurationPtr(CSourceConfiguration::EnumSourceType sourceType) const
	{
		TSourceConfigurationPtr sourceConfigurationPtr;
		TSourceConfigurationPtrMap::const_iterator itSource = this->m_sourceConfigurationPtrMap.find(sourceType);
		if (itSource != this->m_sourceConfigurationPtrMap.end())
			sourceConfigurationPtr = itSource->second;
		return sourceConfigurationPtr;
	}

	void MarketDataReceiverConfiguration::parseConfiguration()
	{
		const std::string& configurationFilePath = g_configurationFilePath;
		FAST::Element* pRootElement = FAST::Element::get_root(configurationFilePath);
		assert(pRootElement != NULL);

		std::string strConfigName = pRootElement->attribute(XML_ATTRIBUTE_CONFIGNAME);	
		GW_INFO_FORMAT("Reading UMDF Configuration name: %s", strConfigName.c_str());

		std::string strPriorityClass = pRootElement->attribute(XML_ATTRIBUTE_STARTUPPRIORITY);
		std::transform(strPriorityClass.begin(), strPriorityClass.end(), strPriorityClass.begin(), tolower);

		std::string templateFilePath = pRootElement->attribute(XML_ATTRIBUTE_TEMPLATEFILEPATH);
		std::string camlConfigFilePath = pRootElement->attribute(XML_ATTRIBUTE_CAMLCONFIGFILEPATH);
		

		for (FAST::Element::Iterator it = pRootElement->children_begin(); it != pRootElement->children_end(); ++it)
		{
			try
			{
				TChannelConfigurationPtr channelConfigurationPtr(new CChannelConfiguration(*it, templateFilePath, camlConfigFilePath));
				this->m_channelConfigurationPtrVector.push_back(channelConfigurationPtr);
			}
			catch (CChannelConfigurationException& cce)
			{
				GW_WARNING_FORMAT(_T("%s(line %d) - %s: Configuration exception: %s."), 
					__FILE__ % __LINE__ % __FUNCTION__ % cce.what());
				continue;
			}
		}
	}

}