// UMDFMarketDataReceiver.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"

#include "UMDFMarketDataReceiver.h"

#include <FASTDecoder.h>
#include <EventDecoder.h>
#include <Element.h>
#include "MarketDataMessageEventHandler.h"
#include <boost/scoped_ptr.hpp>

#include "NanosTimestamp.hpp"

#ifdef _MANAGED
#pragma managed(push, off)
#endif

extern bool _dumpMessages;
extern std::string _dumpMessagesPath;
extern int _dumpSources[4];

NanosTimestamp _ts;

namespace BmfMessagingPlugin
{

	const char* const INCREMENTAL_MULTICAST_GROUP = "IncrementalMulticastGroup";
	const char* const INCREMENTAL_UDP_PORT = "IncrementalUdpPort";
	const char* const INCREMENTAL_UDP_RECV_BUFFER_SIZE = "IncrementalUdpRecvBufferSize";

	const char* const SNAPSHOT_MULTICAST_GROUP = "SnapshotMulticastGroup";
	const char* const SNAPSHOT_UDP_PORT = "SnapshotUdpPort";
	const char* const SNAPSHOT_UDP_RECV_BUFFER_SIZE = "SnapshotUdpRecvBufferSize";

	const char* const FAST_TEMPLATE_FILE = "Template";
	const char* const NO_WORKER_THREADS = "NoWorkerThreads";

	// --------------------------------------------------
	// UMDFMarketDataReceiver implementation
	// --------------------------------------------------
	CUMDFMarketDataReceiver::CUMDFMarketDataReceiver(TChannelConfigurationPtr channelConfigurationPtr)
		: m_channelConfigurationPtr(channelConfigurationPtr), _lastTime(0)
	{
		GW_INFO("Loading CUMDFMarketDataReceiver...");

		this->m_channelID = m_channelConfigurationPtr->getNumber();

		GW_INFO("CUMDFMarketDataReceiver created.");
	}


	CUMDFMarketDataReceiver::~CUMDFMarketDataReceiver(void)
	{
		
	}

	// Sets up threading environment
	void CUMDFMarketDataReceiver::start(HANDLE hStopEvent)
	{
		GW_STACK_PUSH_POP
		// Using external stop handle
		this->m_hStopEvent = hStopEvent;

		// Sets up the signals
		LPSECURITY_ATTRIBUTES lpEventAttributes = NULL;
		LPTSTR lpName = NULL;

		BOOL bInitialState = FALSE;
		BOOL bManualReset = FALSE;

		// Sets up the threads
		LPSECURITY_ATTRIBUTES lpThreadAttributes = NULL;
		unsigned int stackSize = 0;
		LPVOID lpParameter = this;
		unsigned int creationFlags = 0; // 0 - start immediately, CREATE_SUSPENDED - start after explicit ResumeThread


		GW_TRACE("Creating UMDF pool of decoders for instrument definition");
		// ------------------------------------------------------------
		// Create pool of decoders for instrument definition.
		// ------------------------------------------------------------
		std::size_t instrumentDefinitionNoWorkerThreads = this->m_channelConfigurationPtr->getSourceConfigurationPtr(CSourceConfiguration::InstrumentReplay)->getNoWorkerThreads();
		std::vector<FAST::TEventDecoderPtr> instrumentDefinitionDecoderVector;
		//for (std::size_t i = 0; i < instrumentDefinitionNoWorkerThreads; ++i)
		FAST::TEventDecoderPtr decoderPtr;
		{
			FAST::IFIXMessageEventHandlerPtr messageEventHandlerPtr(new CMarketDataMessageEventHandler(this->shared_from_this()));
			std::string templateFilePath(this->m_channelConfigurationPtr->getTemplateFilePath());
			//FAST::TEventDecoderPtr 
			decoderPtr.reset(new FAST::CEventDecoder(templateFilePath));
			decoderPtr->registerEventHandler(messageEventHandlerPtr);
			instrumentDefinitionDecoderVector.push_back(decoderPtr);
		}
		
		if( _dumpSources[0] == 1 ) {
			GW_TRACE("Creating UMDF Multicast instrument definition receiver threads");

			// ------------------------------------------------------------
			// Create Multicast instrument definition receiver threads.
			// ------------------------------------------------------------
			this->m_instrumentDefinitionServicePtr.reset(new boost::asio::io_service(instrumentDefinitionNoWorkerThreads));
			this->m_instrumentDefinitionReceiverPtr.reset(new CMulticastReceiver(
				this->m_channelID, 
				CSourceConfiguration::InstrumentReplay, 
				this->m_instrumentDefinitionServicePtr, 
				this->m_channelConfigurationPtr->getSourceConfigurationPtr(CSourceConfiguration::InstrumentReplay), 
				instrumentDefinitionNoWorkerThreads, 
				decoderPtr));
		}

		//this->m_instrumentDefinitionReceiverPtr->setWaitForFirstMessageInLoop(true);

		GW_TRACE("Creating pool of decoders for market data");
		// ------------------------------------------------------------
		// Create pool of decoders for market data
		// ------------------------------------------------------------
		std::size_t snapshotNoWorkerThreads = this->m_channelConfigurationPtr->getSourceConfigurationPtr(CSourceConfiguration::Snapshot)->getNoWorkerThreads();
		std::size_t incrementalNoWorkerThreads = this->m_channelConfigurationPtr->getSourceConfigurationPtr(CSourceConfiguration::Incremental)->getNoWorkerThreads();

		//std::vector<FAST::TEventDecoderPtr> marketDataDecoderVector;
		//for (std::size_t i = 0; i < (snapshotNoWorkerThreads + incrementalNoWorkerThreads); ++i)
		FAST::TEventDecoderPtr decoderPtr1;
		{
			FAST::IFIXMessageEventHandlerPtr messageEventHandlerPtr(new CMarketDataMessageEventHandler(this->shared_from_this()));
			decoderPtr1.reset(new FAST::CEventDecoder(this->m_channelConfigurationPtr->getTemplateFilePath()));
			decoderPtr1->registerEventHandler(messageEventHandlerPtr);
			//marketDataDecoderVector.push_back(decoderPtr);
		}

		FAST::TEventDecoderPtr decoderPtr3;
		{
			FAST::IFIXMessageEventHandlerPtr messageEventHandlerPtr(new CMarketDataMessageEventHandler(this->shared_from_this()));
			decoderPtr3.reset(new FAST::CEventDecoder(this->m_channelConfigurationPtr->getTemplateFilePath()));
			decoderPtr3->registerEventHandler(messageEventHandlerPtr);
			//marketDataDecoderVector.push_back(decoderPtr);
		}
		
		if( _dumpSources[1] == 1 ) {
			GW_TRACE("Creating UMDF Multicast snapshot receiver threads");
			// ------------------------------------------------------------
			// Create Multicast snapshot receiver threads.
			// ------------------------------------------------------------
			this->m_snapshotServicePtr.reset(new boost::asio::io_service(snapshotNoWorkerThreads));
			this->m_snapshotReceiverPtr.reset(new CMulticastReceiver(
				this->m_channelID, 
				CSourceConfiguration::Snapshot, 
				this->m_snapshotServicePtr, 
				this->m_channelConfigurationPtr->getSourceConfigurationPtr(CSourceConfiguration::Snapshot), 
				snapshotNoWorkerThreads, 
				decoderPtr1));

			//this->m_snapshotReceiverPtr->setWaitForFirstMessageInLoop(false);
		}

		if( _dumpSources[3] == 1 ) {
			GW_TRACE("Creating UMDF Multicast incremental receiver threads");
			// ------------------------------------------------------------
			// Create Multicast incremental receiver threads.
			// ------------------------------------------------------------
			
			TSourceConfigurationPtr sourceConfigurationPtr = this->m_channelConfigurationPtr->getSourceConfigurationPtr(CSourceConfiguration::Incremental);
			if (sourceConfigurationPtr->isUsingCaml())
			{
				GW_ERROR("CAML Disabled in this version.");
				exit(-1);
			}
			else
			{
				this->m_incrementalServicePtr.reset(new boost::asio::io_service(incrementalNoWorkerThreads));
				this->m_incrementalReceiverPtr.reset(new CMulticastReceiver(
					this->m_channelID, 
					CSourceConfiguration::Incremental, 
					this->m_incrementalServicePtr, 
					sourceConfigurationPtr, 
					incrementalNoWorkerThreads, 
					decoderPtr1));
			}

		}

		GW_TRACE("Creating UMDF Multicast receiver threads");

		if( _dumpSources[0] == 1 ) {
			// ------------------------------------------------------------
			// Initialize Multicast receiver threads.
			// ------------------------------------------------------------
			if( m_instrumentDefinitionReceiverPtr.get() != NULL ) {
				boost::thread instrumentDefinitionThread(boost::bind(&IMulticastReceiver::runThread, 
					this->m_instrumentDefinitionReceiverPtr, 1));
			}
		}

		if( _dumpSources[1] == 1 ) {
			// Activate incremental endpoint
			GW_INFO("Activating snapshot endpoint");
			if( m_snapshotReceiverPtr.get() != NULL ) 
			{
				boost::thread incrementalThread(boost::bind(&IMulticastReceiver::runThread, this->m_snapshotReceiverPtr, 1));
			}
		}

		if( _dumpSources[3] == 1 ) {
			// Activate incremental endpoint
			GW_INFO("Activating incremental endpoint");
			if( m_incrementalReceiverPtr.get() != NULL ) 
			{
				boost::thread incrementalThread(boost::bind(&IMulticastReceiver::runThread, this->m_incrementalReceiverPtr, 1));
			}
		}

	}


	// Stop threading environment
	void CUMDFMarketDataReceiver::stop(void)
	{
		if( this->m_incrementalServicePtr.get() != NULL ) this->m_incrementalServicePtr->stop();
		if( this->m_snapshotServicePtr.get() != NULL ) this->m_snapshotServicePtr->stop();
		if( this->m_instrumentDefinitionServicePtr.get() != NULL ) this->m_instrumentDefinitionServicePtr->stop();
		GW_WARNING_FORMAT(_T("%s(%d): Stopped!"), __FUNCTION__ % this->m_channelID);
	}
	
	// Insert the message into the list, sort and notify the consumer thread
	void CUMDFMarketDataReceiver::dispatchMessage(int channel, int source, const std::string &msg)
	{
		__int64 time = _ts.getTimestamp();
		if( _lastTime == 0 ) _lastTime = time;
		
		//TODO: Logar a mensagem
		if( _dumpMessages && _dumpMessagesPath.size() > 0 )
		{
			char fbuf[_MAX_PATH];
			sprintf(fbuf, "%s\\%d_%d_fix.log", _dumpMessagesPath.c_str(), channel, source);
			FILE *f = fopen(fbuf, "ab+");
			if( f != NULL ) {
				fprintf(f, "[%I64d] %s\r\n", (time-_lastTime), msg.c_str());
				fclose(f);
			}
		}

		_lastTime = time;
	
	}

}
